@@ -3,19 +3,28 @@ use warnings;
 use strict;
 use Carp;
 
+use Maypole 2;
+use Apache::MVC 2;
+
 use base 'Apache::MVC';
 
 Maypole::Config->mk_accessors( 'masonx' );
 
 __PACKAGE__->mk_classdata( 'mason_ah' );
 
+use Maypole::Constants;
+
 =head1 NAME
 
 MasonX::Maypole - use Mason as the frontend and view for Maypole version 2
 
+=head1 VERSION
+
+Version 0.02_05
+
 =cut
 
-our $VERSION = '0.4_1';
+our $VERSION = '0.21';
 
 =head1 SYNOPSIS
 
@@ -25,49 +34,51 @@ our $VERSION = '0.4_1';
 
     use Class::DBI::Loader::Relationship;
 
-    use Maypole::Application qw( -Debug2 MasonX AutoUntaint );
+    use MasonX::Maypole 0.2;
+    use base 'MasonX::Maypole';
 
-    BeerDB->setup( 'dbi:mysql:beerdb', 'username', 'password' );
+    BeerDB->setup( 'dbi:mysql:beerdb' );
 
-    BeerDB->config->{template_root}  = '/home/beerdb/www/www/htdocs';
-    BeerDB->config->{uri_base}       = '/';
+    BeerDB->config->{view}           = 'MasonX::Maypole::View';
+    BeerDB->config->{template_root}  = '/var/www/beerdb';
+    BeerDB->config->{uri_base}       = '/beerdb';
     BeerDB->config->{rows_per_page}  = 10;
     BeerDB->config->{display_tables} = [ qw( beer brewery pub style ) ];
-    BeerDB->config->{application_name} = 'The Beer Database';
 
-    BeerDB->config->masonx->{comp_root}  = [ [ factory => '/var/www/maypole/factory' ] ];
-    BeerDB->config->masonx->{data_dir}   = '/home/beerdb/www/www/mdata';
-    BeerDB->config->masonx->{in_package} = 'BeerDB::TestApp';
+    BeerDB->config->masonx->{comp_root}  = [ factory => '/var/www/maypole/factory' ];
+    BeerDB->config->masonx->{data_dir}   = '/path/to/mason/data_dir';
+    BeerDB->config->masonx->{in_package} = 'My::Mason::App';
 
-    BeerDB->auto_untaint;
+    BeerDB::Brewery->untaint_columns( printable => [qw/name notes url/] );
+
+    BeerDB::Style->untaint_columns( printable => [qw/name notes/] );
+
+    BeerDB::Beer->untaint_columns(
+        printable => [qw/abv name price notes/],
+        integer => [qw/style brewery score/],
+        date => [ qw/date/],
+    );
 
     BeerDB->config->{loader}->relationship($_) for (
-        'a brewery produces beers',
-        'a style defines beers',
-        'a pub has beers on handpumps',
-        );
+        "a brewery produces beers",
+        "a style defines beers",
+        "a pub has beers on handpumps");
 
     1;
 
-=head1 DESCRIPTION
-
-A frontend and view for Maypole, using Mason.
-
 =head1 DEVELOPER RELEASE
 
-This version supports HTML::Mason 1.29_1, and should not be used with earlier releases. 
+This release is fundamentally different from previous MasonX::Maypole releases,
+and B<will break sites> that are using them. Previous releases did not work
+with Maypole 2. This and future releases will not work with Maypole 1.
 
-=head1 EXAMPLES
+=head1 DESCRIPTION
 
-Example C<BeerDB.pm> and a C<httpd.conf> VirtualHost setup are included in 
-the C</doc> directory of the distribution.
-
-A working example of the BeerDB application is at C<http://beerdb.riverside-cms.co.uk>, 
-including the C<BeerDB.pm> and C<httpd.conf> used for that site.
+A frontend and view for Maypole 2, using Mason.
 
 =head1 CONFIGURING MASON
 
-Set any parameters for the Mason ApacheHandler in C<<BeerDB->config->{masonx}>>.
+Set any parameters for the Mason ApacheHandler in C<My::Maypole::App->config->{masonx}>.
 This is where to tell Maypole/Mason where the factory templates are stored.
 
 Note that the user the server runs as must have permission to read the files in the
@@ -76,11 +87,6 @@ templates must be readable and executable (i.e. openable). If Mason can't read
 these templates, you may get a cryptic 'file doesn't exist' error, but you
 will not get a 'not permitted' error.
 
-=head1 Maypole::Application
-
-L<Maypole::Application|Maypole::Application> needs to be patched before it will work 
-with MasonX::Maypole. You can download a patched copy from C<http://beerdb.riverside-cms.co.uk>, 
-until the required updates are included in the version distributed with L<Maypole>.
 
 =head1 TEMPLATES
 
@@ -90,8 +96,7 @@ a header and footer to every page, while the dhandler loads the template
 specified in the Maypole request object.
 
 So if you set the factory comp_root to point at the Maypole factory templates,
-the thing should Just Work right out of the box. Except for maypole.css, which 
-you will need to copy to the right place on your server. 
+the thing should Just Work right out of the box.
 
 =head1 METHODS
 
@@ -119,13 +124,10 @@ sub init {
     $mason_cfg->{in_package}   ||= 'HTML::Mason::Commands';
 
     # this provides dynamic table-name component roots
-    $mason_cfg->{dynamic_comp_root} = 1;
-    #$mason_cfg->{request_class}  = 'MasonX::Request::ExtendedCompRoot';
-    #$mason_cfg->{resolver_class} = 'MasonX::Resolver::ExtendedCompRoot';
+    $mason_cfg->{request_class}  = 'MasonX::Request::ExtendedCompRoot';
+    $mason_cfg->{resolver_class} = 'MasonX::Resolver::ExtendedCompRoot';
 
     $class->mason_ah( MasonX::Maypole::ApacheHandler->new( %{ $mason_cfg } ) );
-    
-    $class->config->view || $class->config->view( 'MasonX::Maypole::View' );
 
     $class->SUPER::init;
 }
@@ -173,8 +175,6 @@ CROOT:  foreach my $index ( 0 .. $#$comp_roots )
     push @$comp_roots, [ factory => $factory->[1] || File::Spec->catdir( $template_root, 'factory' ) ];
 
     $class->config->masonx->{comp_root} = $comp_roots;
-    
-    #use Data::Dumper; warn "Base comp roots: " . Dumper( $comp_roots );
 }
 
 =item parse_args
@@ -194,23 +194,45 @@ sub parse_args {
     $self->{query}  = $args;
 }
 
+=item parse_location
+
+This method is B<not> implemented here, but in L<Apache::MVC|Apache::MVC>.
+However, the method there assumes your Maypole app is configured in its
+own C<Location> directive in the Apache config file. Here's a method that
+instead uses the C<base_url> Maypole config parameter. Put it in your Maypole
+class if you need it:
+
+    sub parse_location {
+        my ( $self ) = @_;
+
+        my $uri = $self->ar->uri;
+
+        # Apache::MVC uses $self->ar->location here
+        my $base = $self->config->uri_base;
+
+        ( my $path = $uri ) =~ s/^($base)?\///;
+
+        $self->path( $path );
+
+        $self->parse_path;
+        $self->parse_args;
+    }
+
+
 =item send_output
 
 Template variables have already been exported to Mason components namespace
-in C<MasonX::Maypole::View::template>. This method now runs the Mason C<exec>
+in C<MasonX::Maypole::View::template>. This method now runs the Mason Cexec>
 phase to generate and send output.
 
 =cut
 
 sub send_output {
     my ( $self ) = @_;
-    
+
     # if there was an error, there may already be a report in the output slot,
     # so send it via Apache::MVC
-    if ( $self->output )
-    {
-        return $self->SUPER::send_output if $self->output;
-    }
+    return $self->SUPER::send_output if $self->output;
 
     my $m = eval { $self->mason_ah->prepare_request( $self->ar ) };
 
@@ -233,47 +255,16 @@ sub send_output {
         ? $self->content_type . "; charset=" . $self->document_encoding
         : $self->content_type
     );
-    
-    foreach ( $self->headers_out->field_names ) 
-    {
-        next if /^Content-(Type|Length)/;
-        $self->{ar}->headers_out->set( $_ => $self->headers_out->get( $_ ) );
-    }
-    
-    # I think Mason will do this:
-    #$self->{ar}->send_http_header;
-    
-    my @default_comp_roots = @{ $m->interp->comp_root };
 
     # add dynamic comp root for table queries
-    # Changed to using model_class instead of table in 0.219 ( see Maypole::View::Base::paths() 
-    # - ideally, this stuff would go in a paths() method).
-    if ( $self->model_class )
+    if ( $self->table )
     {
-        my $model_comp_root = File::Spec->catdir( $self->get_template_root, $self->model_class->moniker );
-        # Even with this ugliness, there might be potential for non-unique prefixes, if a site was 
-        # running multiple Maypole sub-apps all using the same Mason datadir...
-        #$m->prefix_comp_root( 'MAYPOLE_MODEL_' . $self->model_class->moniker . "=>$model_comp_root" ) if -d $model_comp_root;
-        
-        # often this path will not exist
-        $m->interp->comp_root( [ [ 'MAYPOLE_MODEL_' . $self->model_class->moniker => $model_comp_root ],
-                                 @default_comp_roots,
-                                 ]
-                               ) if -d $model_comp_root;
+        my $table_comp_root = File::Spec->catdir( $self->get_template_root, $self->table );
+        $m->prefix_comp_root( "table=>$table_comp_root" ) if -d $table_comp_root;
     }
-    
-    #if ( $self->table )
-    #{
-    #    my $table_comp_root = File::Spec->catdir( $self->get_template_root, $self->table );
-    #    $m->prefix_comp_root( "table=>$table_comp_root" ) if -d $table_comp_root;
-    #}
-    
-    warn "Comp roots:\n" . join( "\n", map { "@$_" } @{ $m->interp->comp_root } ) if $self->debug;
 
     # now generate output
     $m->exec;
-    
-    $m->interp->comp_root( [ @default_comp_roots ] );
 }
 
 =item get_template_root
@@ -306,20 +297,6 @@ sub get_template_root { $_[0]->config->template_root }
     }
 }
 
-=item get_request
-
-Replaces C<Apache::MVC::get_request>, using C<Apache::Request::instance()> instead 
-of C<Apache::Request::new()> to obtain the APR object. Calling C<new> means Mason 
-and Maypole have different APR objects, and the Mason one doesn't have any POST 
-data.
-
-=cut
-
-sub get_request {
-    my ( $self, $r ) = @_;
-    $self->{ar} = Apache::Request->instance($r); 
-}
-
 =back
 
 =head1 AUTHOR
